      SUBROUTINE crs_o3(crs, error, first, lambda, n_lambda, ntemp, 
     $     path, temper, type)  
*
*
*BOD
*DESCRIPTION
* Calculates ozone cross sections as a function of
* wavelength and temperature.
*USAGE
**FORTRAN
*     CALL crs_o3( crs, error, lambda, n_lambda, ntemp, path,
*    $             temper, type)  
**PERL
*INPUT
**first
*     logical
*      If true, temperature coefficients for ozone will be 
*      calculated at wavelengths lambda. If first is true, it will
*      be returned as false, just in case you should forget.
*      If false, the temperature coefficients from earlier call will be
*      used. 
*      first should be true the first time crs_o3 is called. It should 
*      also be true if lambda is changed. It does not need to be true
*      if temper is changed.
**lambda
*     real array(*)
*     Wavelengths (nm) at which the ozone cross section is to be calculated.
**n_lambda
*     integer
*     Number of wavelengths lambda.
**ntemp
*     integer
*     Number of different temperatures (levels in the atmosphere) 
**path
*     character string
*     The location of the Radtran data files, e.g. 
*     /usr/local/lib/Radtran/data
**temper
*     real array(*)
*     The temperatures (levels in the atmosphere) at which
*     the ozone cross section is to be calculated.
**type
*     integer
*     The ozone cross section data to be used:
***1     Molina and Molina (1986)
***2     Daumont et al. (1992), Malicet et al. (1995)
***3     Paur and Bass (1985)
*OUTPUT
**crs
*     real array(temper,lambda)
*     The ozone cross section as a function of temper and lambda.
**error
*     integer
*     Error flag. Error checking performed only if first is true.
***0     Everything ok.
***1     Error occurred during reading of data file.
***2     n_lambda == 0 and ntemp != 0
***3     n_lambda != 0 and ntemp == 0
*ERROR_MESSAGES
*ROUTINES_CALLED
*FILES
*     crs_o3_mol_cf.dat           Reference 1 data file
*     crs_o3_dau_cf.dat           Reference 2 data file
*     crs_o3_pab_cf.dat           Reference 3 data file
*     consts.inc               Include file
*     param.inc                Include file
*REFERENCES
*     (1) Molina and Molina, 'Absolute Absorption Cross Sections of
*         Ozone in the 185- to 350-nm wavelength Range', JGR, vol 91,
*         pp. 14,501-14,508, 1986.
*
*     (2) Daumont D, Brion J, Charbonnier J and Malicet J, 
*         "Absorption Cross-Section at Room Temperature",
*         J.Atmos.Chem. no. 15, vol. 145 (1992)
*
*     (3) Paur and Bass, (1985) 'The ultraviolet cross sections of ozone, II, 
*         results and temperature dependence', in: Atmospheric Ozone, proceedings 
*         of the Quadrennial Ozone Symposium, Halkidiki, Greece. 
*         C. Zerefos and A. Ghazi (eds), pp.611-616, D. Reidel, Hingham, MA 
*
*EOD
*
*
*     References:
*     
*     WMO (1986), 'Atmospheric Ozone 1985', WMO Report No. 16.
*
*     
      INCLUDE 'param.inc'
*
      CHARACTER*(*) path
      INTEGER  error,  n_lambda, ntemp, type
      LOGICAL first
*
      REAL crs(0:maxcly, maxpro, *), lambda(*), temper(0:*)
*
* Internal variables
*     
      CHARACTER*21 filenm    ! No. of characters must equal
                             ! length of filename.
      CHARACTER*100 fullname, line
      INTEGER n_crs
      REAL fact, lambda_raw(maxwvnraw),
     $     coeff_crs(NCRSTEMP,maxwvnraw)
      real a, b, gradient

*
      INTEGER closest_above, closest_below
*
      INCLUDE 'consts.inc'
*      SAVE  n_crs, fact, lambda_raw, coeff_crs
*
*     This option pertains to a case where the o3 cross-sections are 
*     given from a fitted formula to data taken by Paur and Bass(1985)
*     
*     Calculate ozone cross sections 
*     
      error = 0

      IF ( first ) THEN
         n_crs    = 0

         CALL zeroit( crs, (maxcly+1)*maxwvn*maxpro)
*     
         IF ( ( n_lambda .LE. 0 ) .AND. ( ntemp .NE. 0 ) ) THEN
            error = 2
            RETURN
         ELSE IF ( ( n_lambda .LE. 0 ) .AND. ( ntemp .NE. 0 ) ) THEN
            error = 3
            RETURN
         ENDIF
*     
         IF ( type .EQ. 1 ) THEN
            filenm   = 'crs/crs_o3_mol_cf.dat'
            n_coeff = 3
            fact     = 1.E-20
         ENDIF
*     
         IF ( type .EQ. 2 ) THEN
            filenm   = 'crs/crs_o3_dau_cf.dat'
            n_coeff = 3
            fact     = 1.E-20
         ENDIF
*     
         IF ( type .EQ. 3 ) THEN
            filenm   = 'crs/crs_o3_pab_cf.dat'
            n_coeff = 3
            fact     = 1.E-20
         ENDIF
*     
         fullname = path//filenm
         OPEN(UNIT=iunit,FILE=fullname,STATUS='OLD',
     $        FORM='FORMATTED',ERR=89, IOSTAT=ios)
         GOTO 881
 89      CONTINUE
         WRITE(*,'(A,A)') 'Error during read of file: ', fullname
         WRITE(*,'(A, i4)') 'IOSTAT: ', ios
         error = 1
         RETURN
 881     CONTINUE
*     
         DO WHILE ( .TRUE. ) 
            READ(iunit,'(A)',END=88 ) line
*     
*     Read all the no-comment lines
*     
            IF ( line(:1) .NE. comcha ) THEN
               n_crs = n_crs + 1
               READ(line,*) lambda_raw(n_crs),   
     $              (coeff_crs(it, n_crs),it=1, n_coeff)
            ENDIF
         ENDDO
 88      CONTINUE
         n_crs = n_crs - 1

         CLOSE(iunit)
         
      ENDIF

      do iv=1, n_lambda
         ia=closest_above(lambda(iv), lambda_raw, n_crs)
         ib=closest_below(lambda(iv), lambda_raw, n_crs)
         if ((ia .gt. n_crs) .or. (ib .lt. 1)) then
            write(*,*) 'crs_o3, cannot handle this case yet'
            write(*,*) 'lambda(iv), n_crs, ia, ib'
            write(*,*) lambda(iv), n_crs, ia, ib
            stop
         endif
         DO it = 0, ntemp
            a = coeff_crs(1,ia) + 
     $           coeff_crs(2,ia) * temper(it) + 
     $           coeff_crs(3,ia) * temper(it)**2.
            if (ia .eq. ib) then
               crs(it, io3, iv) = a*fact
            else 
               b = coeff_crs(1,ib) + 
     $              coeff_crs(2,ib) * temper(it) + 
     $              coeff_crs(3,ib) * temper(it)**2.
               gradient=(a-b)/(lambda_raw(ia)-lambda_raw(ib))
               crs(it, io3, iv) =  
     $              (gradient*(lambda(iv)-lambda_raw(ib))+b) * fact
            endif
         ENDDO
      ENDDO
      RETURN
      END
